from .verify_actor import VerifyActorRunner, format_verify_actor_result


async def test_verify_actor():
    runner = VerifyActorRunner(applications=[])

    result = await runner.run()

    assert result == []


def test_format():
    entry_data = {
        "alice": {"get_actor": False, "post_inbox": False},
        "bob": {"get_actor": True, "post_inbox": True},
        "claire": {"get_actor": True, "post_inbox": True},
        "dean": {"get_actor": False, "post_inbox": False},
        "emily": {"get_actor": True, "post_inbox": True},
        "frank": {"get_actor": True, "post_inbox": True},
    }

    result = [
        {
            "application_name": "test",
            "verify_actor_table": entry_data,
            "messages": "bla",
        }
    ]

    markdown = format_verify_actor_result(result)

    print()
    print()
    print(markdown)



import logging
import json
import click

from quart import Quart, request, render_template, redirect

from .data_provider import DataProvider
from .types import Message
from .server import BlueprintForActors
from .server.verify_actor import ActorVerifier

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@click.command()
@click.option("--only_generate_config", default=False, is_flag=True)
@click.option("--port", default=2909, help="port to run on")
@click.option(
    "--domain", default="localhost:2909", help="domain the service is running on"
)
def verify_actor(only_generate_config, port, domain):
    dp = DataProvider.generate_and_load(only_generate_config)
    app = Quart(__name__)

    actor_list = dp.possible_actors + [dp.one_actor]
    app.register_blueprint(BlueprintForActors(actor_list).blueprint)

    @app.get("/static/styles.css")
    async def stylesheet():
        return await render_template("styles.css"), 200, {"content-type": "text/css"}

    @app.get("/")
    async def index():
        return await render_template("index.html.j2")

    @app.post("/")
    async def verify():
        form_data = await request.form
        actor_uri = form_data.get("actor_uri").strip()
        if not actor_uri:
            return redirect("/")

        message = Message()
        message.add(f"Got Actor Uri {actor_uri}")

        verifier = ActorVerifier(
            actor_list=actor_list, remote_uri=actor_uri, message=message, domain=domain
        )

        result = await verifier.verify()

        if "json" in request.headers.get("accept"):
            return {
                "result": result,
                "messages": json.dumps(message.response, indent=2),
            }

        warnings = []

        if result["alice"]["post_inbox"]:
            result["alice"]["warning"] = True
            warnings.append(
                "Alice should not be able to post to the inbox without \
signing her request."
            )

        if not result["bob"]["get_actor"]:
            result["bob"]["warning"] = True
            warnings.append("Bob should be able to retrieve the actor")
        if not result["claire"]["get_actor"]:
            result["claire"]["warning"] = True
            warnings.append("Claire should be able to retrieve the actor")

        if not result["bob"]["post_inbox"]:
            result["bob"]["warning"] = True
            warnings.append("Bob should be able to post to the inbox")
        if not result["claire"]["post_inbox"]:
            result["claire"]["warning"] = True
            warnings.append(
                """Claire should be able to post to the inbox.
If bob can successfully post to the inbox, this is likely due to using
an unsigned request to retrieve the public key."""
            )

        return await render_template(
            "verify_actor_result.html.j2",
            messages=json.dumps(message.response, indent=2),
            result=result,
            actor_uri=actor_uri,
            warnings=warnings,
            has_warnings=len(warnings) > 0,
        )

    app.run(port=port, host="0.0.0.0", use_reloader=False)


if __name__ == "__main__":
    verify_actor()



import pytest
import json
from datetime import datetime

from fediverse_pasture.data_provider import DataProvider, bovine_actor_for_actor_data

from . import ActivitySender


@pytest.fixture
def activity_sender():
    dp = DataProvider.generate(with_possible_actors=False)

    bovine_actor, actor_object = bovine_actor_for_actor_data(
        "http://localhost/actor", dp.one_actor
    )

    yield ActivitySender.for_actor(bovine_actor, actor_object)


def test_activity_sender(activity_sender):
    activity_sender.init_create_note(lambda x: {**x, "content": "text"})

    assert isinstance(activity_sender.published, datetime)
    assert isinstance(activity_sender.activity, dict)

    obj = activity_sender.activity.get("object")

    assert obj.get("content") == "text"

    text = json.dumps(activity_sender.activity)

    assert len(text) > 20



from dataclasses import dataclass
from datetime import datetime

from bovine.clients.bearer import BearerAuthClient

from fediverse_pasture.types import ApplicationAdapterForLastActivity


@dataclass
class MastodonApplication:
    domain: str
    access_token: str
    username: str
    client: BearerAuthClient | None = None

    def actor_uri(self):
        return f"http://{self.domain}/users/{self.username}"

    async def top_public(self):
        response = await self.client.get(
            f"http://{self.domain}/api/v1/timelines/public"
        )
        public_timeline = await response.json()
        return public_timeline[0]

    async def top_public_with_published(self, published: datetime) -> dict | None:
        data = await self.top_public()
        created_at = data.get("created_at")
        if not created_at:
            return None
        created_at = datetime.fromisoformat(created_at.removesuffix("Z"))
        if created_at == published:
            return data
        return None

    def last_activity(self, session):
        self.client = BearerAuthClient(session, self.access_token)

        return ApplicationAdapterForLastActivity(
            actor_uri=self.actor_uri(),
            fetch_activity=self.top_public_with_published,
            application_name="mastodon",
        )
